@@ -148,7 +148,7 @@
 #endif
 
 /* Macros to provide type safety for callback functions' context parameters */
-#if ((__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ > 3)) && defined(HAVE_TYPEOF))
+#ifdef HAVE_TYPE_CHECKS
 #  define CALLBACK_TYPECHECK(callback, type) \
 	(COMPILE_ERROR_IF_TRUE(!__builtin_types_compatible_p( \
 		typeof(&callback), type)) ? 1 : 0)
@@ -163,7 +163,7 @@
 #  define COMPILE_ERROR_IF_TRUE(condition) 0
 #endif
 
-#if (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ > 0)) && !defined(__cplusplus) && defined(HAVE_TYPEOF)
+#ifdef HAVE_TYPE_CHECKS
 #  define COMPILE_ERROR_IF_TYPES_NOT_COMPATIBLE(_a, _b) \
 	COMPILE_ERROR_IF_TRUE( \
 		!__builtin_types_compatible_p(typeof(_a), typeof(_b)))
@@ -184,6 +184,12 @@
 #  define likely(expr) expr
 #endif
 
+#if defined(__clang__) && ((__clang_major__ > 4) || (__clang_major__ == 3 && __clang_minor__ >= 9))
+#  define ATTR_UNSIGNED_WRAPS __attribute__((no_sanitize("integer")))
+#else
+#  define ATTR_UNSIGNED_WRAPS
+#endif
+
 /* Provide macros for error handling. */
 #ifdef DISABLE_ASSERTS
 #  define i_assert(expr)
@@ -208,8 +214,11 @@
 
 #endif
 
+/* Close the fd and set it to -1. This assert-crashes if fd == 0. Normally
+   this would happen only if an uninitialized fd is attempted to be closed,
+   which is a bug. */
 #define i_close_fd(fd) STMT_START {  \
-	i_assert(*fd != -1); \
+	i_assert(*fd > 0); \
 	if (unlikely(close_keep_errno(fd) < 0)) \
 		i_error("close(%d[%s:%d]) failed: %m", \
 			*(fd), __FILE__, __LINE__); \
@@ -226,4 +235,13 @@
 #  define DOVECOT_PREREQ(maj, min) 0
 #endif
 
+#ifdef __cplusplus
+#  undef STATIC_ARRAY
+#  define STATIC_ARRAY
+#endif
+
+/* Convenience wrappers for initializing a struct */
+#define i_zero(p) memset(p, 0, sizeof(*(p)))
+#define i_zero_safe(p) safe_memset(p, 0, sizeof(*(p)))
+
 #endif
